"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var type_safe_state_js_1 = require("type-safe-state-js");
var rx_utilities_js_1 = require("rx-utilities-js");
/**
 * Act as a centralized storage for state. This store can dispatch actions that
 * will be reduced onto the existing state.
 * @implements {Type} Type implementation.
 */
var Self = /** @class */ (function () {
    function Self() {
        this.action = new rx_utilities_js_1.IncompletableSubject(new rxjs_1.BehaviorSubject(undefined));
        this.state = new rxjs_1.BehaviorSubject(type_safe_state_js_1.State.empty());
        this.subscription = new rxjs_1.Subscription();
    }
    Object.defineProperty(Self.prototype, "actionTrigger", {
        get: function () {
            return this.action;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Self.prototype, "actionStream", {
        get: function () {
            return this.action.asObservable().pipe(rx_utilities_js_1.mapNonNilOrEmpty(function (v) { return v; }));
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Self.prototype, "stateStream", {
        get: function () {
            return this.state;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Self.prototype, "lastState", {
        get: function () {
            return this.state.value;
        },
        enumerable: true,
        configurable: true
    });
    Self.prototype.initialize = function (reducer) {
        var disposable = this.action.asObservable()
            .pipe(rx_utilities_js_1.mapNonNilOrEmpty(function (v) { return v; }), operators_1.scan(function (acc, action) {
            return reducer(acc, action);
        }, this.state.value), operators_1.observeOn(rxjs_1.queueScheduler))
            .subscribe(this.state);
        this.subscription.add(disposable);
    };
    Self.prototype.deinitialize = function () {
        this.subscription.unsubscribe();
    };
    Self.prototype.dispatch = function (action) {
        this.actionTrigger.next(action);
    };
    return Self;
}());
exports.Self = Self;
/**
 * Create and initialize a dispatch store.
 * @param {Reducer<any>} reducer A Reducer instance.
 * @returns {Self} A dispatch store instance.
 */
function createDefault(reducer) {
    var store = new Self();
    store.initialize(reducer);
    return store;
}
exports.createDefault = createDefault;
//# sourceMappingURL=dispatch.js.map